#include "clip.h"

namespace MaskTools { namespace Clip {

/* Clip */
Clip::Clip()                                       : nFrames(-1)     , nWidth(-1)      , nHeight(-1)        { }
Clip::Clip(int nWidth, int nHeight, int nFrames)   : nFrames(nFrames), nWidth(nWidth)  , nHeight(nHeight)   { }

int Clip::width()       const { return nWidth; }
int Clip::height()      const { return nHeight; }
int Clip::frame_count() const { return nFrames; }

/* Input */
Input::Input()                                        : nOffset(0)      , nIdx(0)   , process(UNUSED)  { }
Input::Input(Process process)                         : nOffset(0)      , nIdx(0)   , process(process) { }
Input::Input(Process process, int nIdx)               : nOffset(0)      , nIdx(nIdx), process(process) { }
Input::Input(Process process, int nIdx, int nOffset)  : nOffset(nOffset), nIdx(nIdx), process(process) { }

Input::operator bool() const { return process != UNUSED; }
Input::operator Process() const { return process; }

/* Inputs */
Inputs::Inputs() { }
Inputs::Inputs(const Input &clip1)
{
   clips[0] = clip1;
}
Inputs::Inputs(const Inputs &clips)
{
   this->clips[0] = clips.clips[0];
   this->clips[1] = clips.clips[1];
   this->clips[2] = clips.clips[2];
   this->clips[3] = clips.clips[3];
}
Inputs::Inputs(const Input &clip1, const Input &clip2)
{
   clips[0] = clip1;
   clips[1] = clip2;
}
Inputs::Inputs(const Input &clip1, const Input &clip2, const Input &clip3)
{
   clips[0] = clip1;
   clips[1] = clip2;
   clips[2] = clip3;
}
Inputs::Inputs(const Input &clip1, const Input &clip2, const Input &clip3, const Input &clip4)
{
   clips[0] = clip1;
   clips[1] = clip2;
   clips[2] = clip3;
   clips[3] = clip4;
}

bool Inputs::IsClip(int nIdx) const
{
   return (clips[0] && clips[0].idx() == nIdx) ||
          (clips[1] && clips[1].idx() == nIdx) ||
          (clips[2] && clips[2].idx() == nIdx) ||
          (clips[3] && clips[3].idx() == nIdx);
}

Input Inputs::clip(int nIdx) const
{
   return clips[nIdx];
}

Inputs &IPOneFrame()          { static Inputs ic(Input(IN_PLACE, 0, 0)); return ic; }
Inputs &OneFrame()            { static Inputs ic(Input(CREATE  , 0, 0), Input(SOURCE, 0, 0)); return ic; }
Inputs &IPTwoFrames()         { static Inputs ic(Input(IN_PLACE, 0, 0), Input(SOURCE, 1, 0)); return ic; }
Inputs &TwoFrames()           { static Inputs ic(Input(CREATE  , 0, 0), Input(SOURCE, 0, 0)  , Input(SOURCE, 1, 0))  ; return ic; }
Inputs &IPThreeFrames()       { static Inputs ic(Input(IN_PLACE, 0, 0), Input(SOURCE, 1, 0)  , Input(SOURCE, 2, 0))  ; return ic; }
Inputs &ThreeFrames()         { static Inputs ic(Input(CREATE  , 0, 0), Input(SOURCE, 0, 0)  , Input(SOURCE, 1, 0)   , Input(SOURCE, 2, 0)); return ic; }
Inputs &IPTemporalOneFrame()  { static Inputs ic(Input(IN_PLACE, 0, 0), Input(SOURCE, 0, -1)); return ic; }

} }