#include "filter.h"

using namespace MaskTools;
using namespace MaskTools::Picture;
using namespace MaskTools::Parameters;
using namespace MaskTools::Constraints;

namespace MaskTools { namespace Filters { namespace Invert {

void Filter::ProcessY(PlaneProcessor *processor, Plane &dst, CPlane &src1, CPlane &src2, CPlane &src3)
{
   static_cast<Processor*>(processor)(dst.plane(), dst.pitch(), nWidth, nHeight);
}

void Filter::ProcessU(PlaneProcessor *processor, Plane &dst, CPlane &src1, CPlane &src2, CPlane &src3)
{
   static_cast<Processor*>(processor)(dst.plane(), dst.pitch(), nWidthUV, nHeightUV);
}

void Filter::ProcessV(PlaneProcessor *processor, Plane &dst, CPlane &src1, CPlane &src2, CPlane &src3)
{
   static_cast<Processor*>(processor)(dst.plane(), dst.pitch(), nWidthUV, nHeightUV);
}

Filter::Filter(const Parameters::Parameters &parameters) : MaskTools::Filter(parameters)
{
   /* add the processors */
   addProcessor(Constraints::Processor<PlaneProcessor>(PPlaneProcessor(&invert_c), Constraint(CPU_NONE, 1, 1, 1, 1), 0));
   addProcessor(Constraints::Processor<PlaneProcessor>(PPlaneProcessor(&Invert_invert8_mmx), Constraint(CPU_MMX, 8, 1, 1, 1), 1));
   addProcessor(Constraints::Processor<PlaneProcessor>(PPlaneProcessor(&Invert_invert8_isse), Constraint(CPU_ISSE, 8, 1, 1, 1), 2));
   addProcessor(Constraints::Processor<PlaneProcessor>(PPlaneProcessor(&Invert_invert8_3dnow), Constraint(CPU_3DNOW, 8, 1, 1, 1), 3));
}

String Filter::getName() { return "mt_invert"; }
Signature Filter::getSignature()
{
   Signature signature;
   signature.push_back(Parameter(CLIP, ""));

   return addDefaultCommons(signature);
}

Clip::Inputs Filter::inputClips = Clip::IPOneFrame();

} } }
