#include "filter.h"
#include "../../../common/parser/parser.h"

using namespace MaskTools;
using namespace MaskTools::Picture;
using namespace MaskTools::Parameters;
using namespace MaskTools::Constraints;
using namespace MaskTools::Parser;

namespace MaskTools { namespace Filters { namespace Mask { namespace Hysteresis {

void Filter::initializeStack()
{
   if ( !pbStack )
      pbStack = new Byte[nWidth * nHeight];
}

void Filter::ProcessY(PlaneProcessor *processor, Plane &dst, CPlane &src1,
                      CPlane &src2, CPlane &src3)
{
   initializeStack();
   static_cast<Processor*>(processor)(dst.plane(), dst.pitch(), src1.plane(), src1.pitch(), src2.plane(), src2.pitch(), pbStack, nWidth, nHeight);
}

void Filter::ProcessU(PlaneProcessor *processor, Plane &dst, CPlane &src1,
                      CPlane &src2, CPlane &src3)
{
   initializeStack();
   static_cast<Processor*>(processor)(dst.plane(), dst.pitch(), src1.plane(), src1.pitch(), src2.plane(), src2.pitch(), pbStack, nWidthUV, nHeightUV);
}

void Filter::ProcessV(PlaneProcessor *processor, Plane &dst, CPlane &src1,
                      CPlane &src2, CPlane &src3)
{
   initializeStack();
   static_cast<Processor*>(processor)(dst.plane(), dst.pitch(), src1.plane(), src1.pitch(), src2.plane(), src2.pitch(), pbStack, nWidthUV, nHeightUV);
}


Filter::Filter(const Parameters::Parameters &parameters) : MaskTools::Filter(parameters), pbStack(NULL)
{
   addProcessor(Constraints::Processor<PlaneProcessor>(PPlaneProcessor(hysteresis_c), Constraint(CPU_NONE, 1, 1, 1, 1), 0));
}

Filter::~Filter()
{
   if ( pbStack )
      delete pbStack;

   pbStack = NULL;
}

String Filter::getName() { return "mt_hysteresis"; }

Signature Filter::getSignature()
{
   Signature signature;
   signature.push_back(Parameter(CLIP, ""));
   signature.push_back(Parameter(CLIP, ""));

   return addDefaultCommons(signature);
}

Clip::Inputs Filter::inputClips = Clip::TwoFrames();

} } } }
