#include "filter.h"

using namespace MaskTools;
using namespace MaskTools::Picture;
using namespace MaskTools::Parameters;
using namespace MaskTools::Constraints;

namespace MaskTools { namespace Filters { namespace Merge {

void Filter::ProcessY(PlaneProcessor *processor, Plane &dst, CPlane &src1,
                      CPlane &src2, CPlane &src3)
{
   static_cast<Processor*>(processor)(dst.plane(), dst.pitch(), src1.plane(), src1.pitch(), src2.plane(), src2.pitch(), nWidth, nHeight);
}

void Filter::ProcessU(PlaneProcessor *processor, Plane &dst, CPlane &src1,
                      CPlane &src2, CPlane &src3)
{
   static_cast<Processor*>(processor)(dst.plane(), dst.pitch(), src1.plane(), src1.pitch(), src2.plane(), src2.pitch(), nWidthUV, nHeightUV);
}

void Filter::ProcessV(PlaneProcessor *processor, Plane &dst, CPlane &src1,
                      CPlane &src2, CPlane &src3)
{
   static_cast<Processor*>(processor)(dst.plane(), dst.pitch(), src1.plane(), src1.pitch(), src2.plane(), src2.pitch(), nWidthUV, nHeightUV);
}


Filter::Filter(const Parameters::Parameters &parameters) : MaskTools::Filter(parameters)
{
   /* add the processors */
   addProcessor(Constraints::Processor<PlaneProcessor>(PPlaneProcessor(merge_c), Constraint(CPU_NONE, 1, 1, 1, 1), 0));
   addProcessor(Constraints::Processor<PlaneProcessor>(PPlaneProcessor(Merge_merge4_mmx), Constraint(CPU_MMX, 4, 1, 1, 1), 1));
   addProcessor(Constraints::Processor<PlaneProcessor>(PPlaneProcessor(Merge_merge8_mmx), Constraint(CPU_MMX, 8, 1, 1, 1), 2));
   addProcessor(Constraints::Processor<PlaneProcessor>(PPlaneProcessor(Merge_merge8_isse), Constraint(CPU_ISSE, 8, 1, 1, 1), 3));
   addProcessor(Constraints::Processor<PlaneProcessor>(PPlaneProcessor(Merge_merge8_3dnow), Constraint(CPU_3DNOW, 8, 1, 1, 1), 4));
}

String Filter::getName() { return "mt_merge"; }

Signature Filter::getSignature()
{
   Signature signature;
   signature.push_back(Parameter(CLIP, ""));
   signature.push_back(Parameter(CLIP, ""));
   signature.push_back(Parameter(CLIP, ""));

   signature.push_back(Parameter(Operator(PROCESS), "Y"));
   signature.push_back(Parameter(Operator(COPY), "U"));
   signature.push_back(Parameter(Operator(COPY), "V"));
   signature.push_back(Parameter(0     , "offX"));
   signature.push_back(Parameter(0     , "offY"));
   signature.push_back(Parameter(-1    , "w"));
   signature.push_back(Parameter(-1    , "h"));
   signature.push_back(Parameter(true  , "mmx"));
   signature.push_back(Parameter(true  , "isse"));
   signature.push_back(Parameter(true  , "sse2"));
   signature.push_back(Parameter(false , "sse3"));
   signature.push_back(Parameter(true  , "d3now"));
   signature.push_back(Parameter(true  , "d3now2"));

   return signature;
}

Clip::Inputs Filter::inputClips = Clip::IPThreeFrames();

} } }
