#include "filter.h"
#include "../../common/parser/parser.h"

using namespace MaskTools;
using namespace MaskTools::Picture;
using namespace MaskTools::Parameters;
using namespace MaskTools::Constraints;
using namespace MaskTools::Parser;

namespace MaskTools { namespace Filters { namespace Morphologic {

void Filter::ProcessY(PlaneProcessor *processor, Plane &dst, CPlane &src1, CPlane &src2, CPlane &src3)
{
   static_cast<Processor*>(processor)(dst.plane(), dst.pitch(), src1.plane(), src1.pitch(), nMaxDeviation, pCoordinates, nCoordinates, nWidth, nHeight);
}

void Filter::ProcessU(PlaneProcessor *processor, Plane &dst, CPlane &src1, CPlane &src2, CPlane &src3)
{
   static_cast<Processor*>(processor)(dst.plane(), dst.pitch(), src1.plane(), src1.pitch(), nMaxDeviationUV, pCoordinates, nCoordinates, nWidthUV, nHeightUV);
}

void Filter::ProcessV(PlaneProcessor *processor, Plane &dst, CPlane &src1, CPlane &src2, CPlane &src3)
{
   static_cast<Processor*>(processor)(dst.plane(), dst.pitch(), src1.plane(), src1.pitch(), nMaxDeviationUV, pCoordinates, nCoordinates, nWidthUV, nHeightUV);
}

void Filter::FillCoordinates(const String &coordinates)
{
   std::list<Symbol> coeffs = Parser::getDefaultParser().parse(coordinates, " (),;.").getExpression();
   nCoordinates = coeffs.size();
   pCoordinates = new int[nCoordinates];
   int i = 0;

   while ( !coeffs.empty() )
   {
      pCoordinates[i++] = int(coeffs.front().getValue(0, 0, 0));
      coeffs.pop_front();
   }
}

Filter::Filter(const Parameters::Parameters &parameters) : MaskTools::Filter(parameters), pCoordinates(NULL), nCoordinates(0)
{
   nMaxDeviation = clip(parameters["thY"].toInt(), 0, 255);
   nMaxDeviationUV = clip(parameters["thC"].toInt(), 0, 255);
}

Filter::~Filter()
{
   if ( pCoordinates )
      delete[] pCoordinates;
   pCoordinates = NULL;
}

Clip::Inputs Filter::inputClips = Clip::OneFrame();

} } }