#include "filter.h"

using namespace MaskTools;
using namespace MaskTools::Picture;
using namespace MaskTools::Parameters;
using namespace MaskTools::Constraints;

namespace MaskTools { namespace Filters { namespace Morphologic { namespace Inpand {

Filter::Filter(const Parameters::Parameters &parameters) : Morphologic::Filter(parameters)
{
   if ( parameters["mode"].toString() == "square" )
   {
      addProcessor(Constraints::Processor<PlaneProcessor>(PPlaneProcessor(inpand_square_c), Constraint(CPU_NONE, 1, 1, 1, 1), 0));
      addProcessor(Constraints::Processor<PlaneProcessor>(PPlaneProcessor(inpand8_isse), Constraint(CPU_ISSE, 8, 1, 1, 1), 1));
      addProcessor(Constraints::Processor<PlaneProcessor>(PPlaneProcessor(inpand8_3dnow), Constraint(CPU_3DNOW, 8, 1, 1, 1), 2));
   }
   else if ( parameters["mode"].toString() == "horizontal" )
   {
      addProcessor(Constraints::Processor<PlaneProcessor>(PPlaneProcessor(inpand_horizontal_c), Constraint(CPU_NONE, 1, 1, 1, 1), 0));
   }
   else if ( parameters["mode"].toString() == "vertical" )
   {
      addProcessor(Constraints::Processor<PlaneProcessor>(PPlaneProcessor(inpand_vertical_c), Constraint(CPU_NONE, 1, 1, 1, 1), 0));
   }
   else if ( parameters["mode"].toString() == "both" )
   {
      addProcessor(Constraints::Processor<PlaneProcessor>(PPlaneProcessor(inpand_both_c), Constraint(CPU_NONE, 1, 1, 1, 1), 0));
   }
   else
   {
      addProcessor(Constraints::Processor<PlaneProcessor>(PPlaneProcessor(inpand_custom_c), Constraint(CPU_NONE, 1, 1, 1, 1), 0));
      FillCoordinates(parameters["mode"].toString());
   }
}

String Filter::getName() { return "mt_inpand"; }

Signature Filter::getSignature()
{
   Signature signature;
   signature.push_back(Parameter(CLIP, ""));
   signature.push_back(Parameter(255, "thY"));
   signature.push_back(Parameter(255, "thC"));
   signature.push_back(Parameter(String("square"), "mode"));

   return addDefaultCommons(signature);
}

} } } }