#include "../../avs25/avisynth.h"
#include "../../common/common.h"
#include "../../helpers/forms/forms.h"
#include "../../helpers/parser/parser.h"
#include "../../helpers/parser/spirit.h"

using namespace MaskTools;
using namespace MaskTools::Helpers::Forms;
using namespace MaskTools::Helpers::PolishConverter;

template<RadiusForm rf>
AVSValue __cdecl CreateRadiusForm(AVSValue args, void *user_data, IScriptEnvironment *env)
{
   return AVSValue((new String(rf(args[0].AsInt(1))))->c_str()); /* grrrrr -> memory leak */
}

template<StringConverter sc>
AVSValue __cdecl CreateStringConverter(AVSValue args, void *user_data, IScriptEnvironment *env)
{
   return AVSValue((new String(sc(args[0].AsString("x"))))->c_str()); /* grrrrr -> memory leak */
}

template<BiRadiusForm bf>
AVSValue __cdecl CreateBiRadiusForm(AVSValue args, void *user_data, IScriptEnvironment *env)
{
   return AVSValue((new String(bf(args[0].AsInt(1), args[1].AsInt(1))))->c_str()); /* grrrrr -> memory leak */
}

template<RadiusForm rf>
static void DeclareRadiusForm(const String &name, IScriptEnvironment *env)
{
   env->AddFunction(name.c_str(), "i", CreateRadiusForm<rf>, NULL);
}

template<StringConverter sc>
static void DeclareStringConverter(const String &name, IScriptEnvironment *env)
{
   env->AddFunction(name.c_str(), "s", CreateStringConverter<sc>, NULL);
}

template<BiRadiusForm bf>
static void DeclareBiRadiusForm(const String &name, IScriptEnvironment *env)
{
   env->AddFunction(name.c_str(), "ii", CreateBiRadiusForm<bf>, NULL);
}

void DeclareHelpers(IScriptEnvironment* env)
{
   DeclareBiRadiusForm<LosangeToString>("mt_losange", env);
   DeclareBiRadiusForm<RectangleToString>("mt_rectangle", env);
   DeclareBiRadiusForm<EllipseToString>("mt_ellipse", env);

   DeclareRadiusForm<SquareToString>("mt_square", env);
   DeclareRadiusForm<CircleToString>("mt_circle", env);
   DeclareRadiusForm<DiamondToString>("mt_diamond", env);

   DeclareStringConverter<Converter>("mt_polish", env);
}